
#include <QGraphicsGridLayout>
#include <QGraphicsLinearLayout>
#include <QGraphicsProxyWidget>
#include <QGraphicsSceneMouseEvent>

#include <QApplication>
#include <QEvent>
#include <QPoint>
#include <QKeyEvent>
#include <QRect>
#include <QDesktopWidget>
#include <QPainter>

#include "clippinglabel.h"
#include "WPData.h"
#include "wpcommentgraphicslist.h"
#include "wpcommentgraphicslistitem.h"
#include "WPNetworkEngine.h"
#include "MDragWidget.h"
#include "themedatastorage.h"
#include "busyindicator.h"
#include "wpcommongeometry.h"

#include "HtmlPainter.h"

WPCommentGraphicsListItem::StandardGeometry WPCommentGraphicsListItem::iStandardGeometry;

WPCommentGraphicsListItem::WPCommentGraphicsListItem(const WPComment aComment, bool aShowFullComment, BusyIndicator& aBusyIndicator, ThemeData& aThemeData)
	: WPGraphicsListItem(aBusyIndicator, aThemeData)
    , iComment(aComment)
    , iShowFullComment(aShowFullComment)
{
    
    iDate = aComment->GetShortLocalTimeString(); 
    
    iTitle = new HtmlPainter(HtmlPainter::Info, true);
    iText = new HtmlPainter(HtmlPainter::Content, true);
    Update(iComment);
}

WPCommentGraphicsListItem::~WPCommentGraphicsListItem()
{
}

void WPCommentGraphicsListItem::SetAvatarImage(const QPixmap &aPixmap)
{
    if (!aPixmap.isNull())
        iPixmap = aPixmap;
}

WPComment WPCommentGraphicsListItem::GetComment() const
{
    return iComment;
}

void WPCommentGraphicsListItem::Update(WPComment aComment)
{
    // XXX: no-op?
    iComment = aComment;
    
    iTitle->setContent(iComment->iPostTitle);
    iText->setContent(iComment->iContent);
}

bool WPCommentGraphicsListItem::operator<(WPCommentGraphicsListItem const & other)
{
    return iComment->GetLocalTime() < other.iComment->GetLocalTime();
}

void WPCommentGraphicsListItem::ChangeViewMode()
{
    prepareGeometryChange();
    iShowFullComment = !iShowFullComment;
    SetViewMode(iShowFullComment);
}

void WPCommentGraphicsListItem::paintContent(QPainter* painter, const QStyleOptionGraphicsItem* option, QWidget* /*widget*/)
{    
    // Common part
    if (!iPixmap.isNull())
    {
        // TODO default avatar from themestorage?
        painter->drawPixmap(iStandardGeometry.iPixmap.topLeft(), iPixmap);
    }
    
    painter->setPen(Qt::black);
    painter->setFont(*iThemeData.iListDetailFont);
    painter->drawText(iStandardGeometry.iAuthor, iComment->iAuthor);
    painter->drawText(iStandardGeometry.iDate, iDate);
    
    if (iShowFullComment)
    {
        iTitle->paint(painter, iExpandedGeometry.iTitle);
        iText->paint(painter, iExpandedGeometry.iComment);
        
        // XXX elide
        painter->setFont(*iThemeData.iListContentFont);
        painter->drawText(iExpandedGeometry.iEmail, iComment->iAuthorEmail);
        painter->drawText(iExpandedGeometry.iWebsite, iComment->iAuthorUrl);
    }
    else
    {
        iTitle->setWidthAndMode(iStandardGeometry.iTitle.width(), true);
        iTitle->paint(painter, iStandardGeometry.iTitle);
        
        iText->setWidthAndMode(iStandardGeometry.iComment.width(), true);
        iText->paint(painter, iStandardGeometry.iComment);
        
        // XXX elide
        painter->setFont(*iThemeData.iListContentFont);
        painter->drawText(iStandardGeometry.iEmail, iComment->iAuthorEmail);
        painter->drawText(iStandardGeometry.iWebsite, iComment->iAuthorUrl);
    }
}

bool WPCommentGraphicsListItem::IsBusy()
{
    return iComment->IsBusy();
}

bool WPCommentGraphicsListItem::HasError()
{
    return iComment->iError;
}

void WPCommentGraphicsListItem::handleGeomeryChange()
{
	SetViewMode(iShowFullComment);
}

void WPCommentGraphicsListItem::SetViewMode(bool aShowFullComment)
{
    if (aShowFullComment)
    {
        iExpandedGeometry.update(*this);
        setMinimumHeight(iExpandedGeometry.height());
    }
    else
    {
        iStandardGeometry.update(*this);
        setMinimumHeight(iStandardGeometry.height());
    }
}

WPCommentGraphicsListItem::StandardGeometry::StandardGeometry()
  : iHeight(0)
    { }

void WPCommentGraphicsListItem::StandardGeometry::update(WPCommentGraphicsListItem & parent)
    {
    if(iInitSize == parent.size()) { return; }
    
    // XXX
    int gravatarSize = 80;
    int padding = parent.iThemeData.iListItemPadding;
    int edgePadding = padding + parent.iThemeData.iListItemShadowMargin;
    
    // Font metrics
    QFontMetrics detailFm(*parent.iThemeData.iListDetailFont);
    
    // Layout rects
    iPixmap.setRect(edgePadding, edgePadding, gravatarSize, gravatarSize);
    iAuthor.setRect(edgePadding, iPixmap.bottom() + padding, gravatarSize, detailFm.height());
    iDate.setRect(edgePadding, iAuthor.bottom() + padding, gravatarSize, detailFm.height());
    
    int remainingWidth = parent.size().width() - iPixmap.right() - (padding + edgePadding);
    int col2x = iPixmap.right() + padding;
    iTitle.setRect(col2x, edgePadding, remainingWidth, parent.iTitle->fontMetrics().height() );
    iComment.setRect(col2x, iTitle.bottom() + 3 * padding, remainingWidth, parent.iText->fontMetrics().height());
    iEmail.setRect(col2x, iComment.bottom() + 3 * padding, remainingWidth, detailFm.height());
    iWebsite.setRect(col2x, iEmail.bottom(), remainingWidth, detailFm.height());
    
    updateHeight(parent.iThemeData);
    iInitSize = parent.size();
    }

void WPCommentGraphicsListItem::StandardGeometry::updateHeight(ThemeData const & theme)
    {
    iHeight = std::max(iDate.bottom(), iWebsite.bottom()) +
              theme.iListItemPadding + theme.iListItemShadowMargin;
    }

void WPCommentGraphicsListItem::ExpandedGeometry::update(WPCommentGraphicsListItem & parent)
    {
    if(iInitSize == parent.size()) { return; }
    
    StandardGeometry::update(parent);
    
    int padding = parent.iThemeData.iListItemPadding;
    const int maxHeight(1000);
    
    iTitle.setSize(parent.iTitle->setWidthAndMode(iTitle.width(), false));
    
    iComment.setSize(parent.iText->setWidthAndMode(iComment.width(), false));
    iComment.moveTop(iTitle.bottom() + 3 * padding);
    
    iEmail.moveTop(iComment.bottom() + 3 * padding);
    iWebsite.moveTop(iEmail.bottom());
    
    updateHeight(parent.iThemeData);
    iInitSize = parent.size();
    }

